@@ -5,21 +5,16 @@ from invisibleroads_macros_security import make_random_string
 
 class DictionarySafe():
 
-    def __init__(
-            self, constant_value_by_key, variable_pack_by_key,
-            variable_key_length):
-        self.constant_value_by_key = constant_value_by_key
-        self.variable_pack_by_key = variable_pack_by_key
-        self.variable_key_length = variable_key_length
+    def __init__(self, pack_by_key, key_length):
+        self.pack_by_key = pack_by_key
+        self.key_length = key_length
 
     def put(self, value, time_in_seconds=None):
+        key_length = self.key_length
         # Use keys() until https://github.com/python/cpython/pull/17333 merges
-        keys = set(
-            self.constant_value_by_key.keys()
-        ) | set(
-            self.variable_pack_by_key.keys())
+        keys = self.pack_by_key.keys()
         while True:
-            key = make_random_string(self.variable_key_length)
+            key = make_random_string(key_length)
             if key not in keys:
                 break
         self.set(key, value, time_in_seconds)
@@ -27,17 +22,17 @@ class DictionarySafe():
 
     def set(self, key, value, time_in_seconds=None):
         expiration_datetime = get_expiration_datetime(time_in_seconds)
-        self.variable_pack_by_key[key] = value, expiration_datetime
+        self.pack_by_key[key] = value, expiration_datetime
 
     def get(self, key):
-        variable_pack_by_key = self.variable_pack_by_key
+        pack_by_key = self.pack_by_key
         try:
-            value, expiration_datetime = variable_pack_by_key[key]
+            value, expiration_datetime = pack_by_key[key]
         except KeyError:
-            value = self.constant_value_by_key[key]
+            raise
         else:
             if expiration_datetime and datetime.now() > expiration_datetime:
-                del variable_pack_by_key[key]
+                del pack_by_key[key]
                 raise KeyError(key)
         return value
 
