@@ -1,9 +1,9 @@
+# TODO: Validate variable view configurations
 import csv
 import json
 import shutil
 from dataclasses import dataclass
 from logging import getLogger
-from string import Template as StringTemplate
 from urllib.request import urlretrieve as download_uri
 
 from importlib_metadata import entry_points
@@ -12,12 +12,10 @@ from invisibleroads_macros_text import format_name, format_slug
 from invisibleroads_macros_web.escape import (
     escape_quotes_html,
     escape_quotes_js)
-from jinja2 import Template as JinjaTemplate
 
 from ..constants import (
     CACHED_FILE_SIZE_LIMIT_IN_BYTES,
     MAXIMUM_FILE_CACHE_LENGTH,
-    TEMPLATES_FOLDER,
     VARIABLE_ID_TEMPLATE_PATTERN,
     VIEW_BY_NAME)
 from ..exceptions import (
@@ -27,6 +25,7 @@ from ..exceptions import (
 from ..macros.disk import FileCache
 from ..macros.iterable import find_item
 from ..macros.package import import_attribute
+from .asset import asset_storage
 from .interface import Batch
 
 
@@ -34,7 +33,7 @@ from .interface import Batch
 class Element():
 
     id: str
-    root_uri: str
+    request_params: str
     mode_name: str
     design_name: str
     for_print: bool
@@ -50,7 +49,6 @@ class VariableView():
         self.variable_definition = variable_definition
         self.variable_id = variable_definition.id
         self.variable_path = variable_definition.path
-        self.mode_name = variable_definition.mode_name
 
     @classmethod
     def get_from(Class, variable_definition):
@@ -73,16 +71,16 @@ class VariableView():
             render = self.render_input
         else:
             render = self.render_output
-        jinja_dictionary = render(b, x)
-        main_text = jinja_dictionary['main_text']
+        page_dictionary = render(b, x)
+        main_text = page_dictionary['main_text']
         if x.design_name != 'none':
             if main_text.endswith('</a>') or main_text.endswith('</span>'):
                 tag_name = 'span'
             else:
                 tag_name = 'div'
-            jinja_dictionary['main_text'] = '<%s class="_view">%s</%s>' % (
+            page_dictionary['main_text'] = '<%s class="_view">%s</%s>' % (
                 tag_name, main_text, tag_name)
-        return jinja_dictionary
+        return page_dictionary
 
     def render_input(self, b: Batch, x: Element):
         return {
@@ -115,7 +113,7 @@ class LinkView(VariableView):
         link_text = c.get('link-text', file_name)
         main_text = (
             f'<a id="{element_id}" href="{data_uri}" '
-            f'class="_{self.mode_name} _{self.view_name} {variable_id}" '
+            f'class="_{x.mode_name} _{self.view_name} {variable_id}" '
             f'download="{escape_quotes_html(file_name)}">'
             f'{link_text}</a>')
         if x.design_name not in ['none']:
@@ -129,11 +127,8 @@ class LinkView(VariableView):
             }),
         ]
         return {
-            'css_uris': [],
-            'js_uris': [],
-            'main_text': main_text,
-            'js_texts': js_texts,
-        }
+            'css_uris': [], 'js_uris': [], 'main_text': main_text,
+            'js_texts': js_texts}
 
 
 class StringView(VariableView):
@@ -144,9 +139,9 @@ class StringView(VariableView):
         'title': str.title,
     }
 
-    def get_value(self, b: Batch):
+    def get_value(self, b: Batch, x: Element):
         variable_definition = self.variable_definition
-        data = b.get_data(variable_definition)
+        data = b.load_data_from(x.request_params, variable_definition)
         if 'value' in data:
             value = data['value']
         elif 'path' in data:
@@ -160,15 +155,16 @@ class StringView(VariableView):
         variable_id = self.variable_id
         view_name = self.view_name
         element_id = x.id
-        value = self.get_value(b)
+        value = self.get_value(b, x)
         c = b.get_variable_configuration(variable_definition)
-        main_text = STRING_HTML_INPUT.substitute({
+        main_text = STRING_HTML_INPUT.render({
             'element_id': element_id,
-            'mode_name': self.mode_name,
+            'mode_name': x.mode_name,
             'view_name': view_name,
             'variable_id': variable_id,
             'value': escape_quotes_html(value),
             'input_type': self.input_type,
+            'suggestions': c.get('suggestions', []),
         })
         if x.design_name not in ['none']:
             main_text = add_label_html(main_text, c, variable_id, element_id)
@@ -176,14 +172,11 @@ class StringView(VariableView):
             STRING_JS_INPUT.substitute({'view_name': view_name}),
         ]
         return {
-            'css_uris': [],
-            'js_uris': [],
-            'main_text': main_text,
-            'js_texts': js_texts,
-        }
+            'css_uris': [], 'js_uris': [], 'main_text': main_text,
+            'js_texts': js_texts}
 
     def render_output(self, b: Batch, x: Element):
-        value = self.get_value(b)
+        value = self.get_value(b, x)
         try:
             value = apply_functions(
                 value, x.function_names, self.function_by_name)
@@ -196,7 +189,7 @@ class StringView(VariableView):
         c = b.get_variable_configuration(variable_definition)
         main_text = (
             f'<span id="{x.id}" '
-            f'class="_{self.mode_name} _{self.view_name} {self.variable_id}">'
+            f'class="_{x.mode_name} _{self.view_name} {self.variable_id}">'
             f'{value}</span>')
         if x.design_name not in ['none']:
             main_text = add_label_html(main_text, c, variable_id, element_id)
@@ -205,15 +198,10 @@ class StringView(VariableView):
             STRING_JS_OUTPUT.substitute({
                 'variable_id': variable_id,
                 'element_id': element_id,
-                'data_uri': data_uri,
-            }),
-        ]
+                'data_uri': data_uri})]
         return {
-            'css_uris': [],
-            'js_uris': [],
-            'main_text': main_text,
-            'js_texts': js_texts,
-        }
+            'css_uris': [], 'js_uris': [], 'main_text': main_text,
+            'js_texts': js_texts}
 
 
 class NumberView(StringView):
@@ -249,7 +237,7 @@ class TextView(VariableView):
 
     def render_input(self, b: Batch, x: Element):
         variable_definition = self.variable_definition
-        data = b.get_data_from_request(variable_definition)
+        data = get_data_from(x.request_params, variable_definition)
         value = data.get('value', '')
         variable_id = self.variable_id
         view_name = self.view_name
@@ -257,30 +245,25 @@ class TextView(VariableView):
         c = b.get_variable_configuration(variable_definition)
         main_text = TEXT_HTML_INPUT.substitute({
             'element_id': element_id,
-            'mode_name': self.mode_name,
+            'mode_name': x.mode_name,
             'view_name': view_name,
             'variable_id': variable_id,
             'attribute_string': '' if value else ' disabled',
-            'value': value,
-        })
+            'value': value})
         if x.design_name not in ['none']:
             main_text = add_label_html(main_text, c, variable_id, element_id)
         js_texts = [
             STRING_JS_HEADER,
-            STRING_JS_INPUT.substitute({'view_name': view_name}),
-            TEXT_JS_HEADER,
-        ]
+            STRING_JS_INPUT.substitute({'view_name': view_name})]
         if not value:
-            js_texts.append(TEXT_JS_INPUT.substitute({
-                'element_id': element_id,
-                'data_uri': b.get_data_uri(variable_definition, x),
-            }))
+            js_texts.extend([
+                TEXT_JS_HEADER,
+                TEXT_JS_INPUT.substitute({
+                    'element_id': element_id,
+                    'data_uri': b.get_data_uri(variable_definition, x)})])
         return {
-            'css_uris': [],
-            'js_uris': [],
-            'main_text': main_text,
-            'js_texts': js_texts,
-        }
+            'css_uris': [], 'js_uris': [], 'main_text': main_text,
+            'js_texts': js_texts}
 
     def render_output(self, b: Batch, x: Element):
         variable_definition = self.variable_definition
@@ -290,24 +273,20 @@ class TextView(VariableView):
         c = b.get_variable_configuration(variable_definition)
         main_text = (
             f'<span id="{x.id}" '
-            f'class="_{self.mode_name} _{self.view_name} {self.variable_id}">'
+            f'class="_{x.mode_name} _{self.view_name} {self.variable_id}">'
             '</span>')
         if x.design_name not in ['none']:
             main_text = add_label_html(main_text, c, variable_id, element_id)
         js_texts = [
             STRING_JS_HEADER,
+            TEXT_JS_HEADER,
             TEXT_JS_OUTPUT.substitute({
                 'variable_id': variable_id,
                 'element_id': element_id,
-                'data_uri': data_uri,
-            }),
-        ]
+                'data_uri': data_uri})]
         return {
-            'css_uris': [],
-            'js_uris': [],
-            'main_text': main_text,
-            'js_texts': js_texts,
-        }
+            'css_uris': [], 'js_uris': [], 'main_text': main_text,
+            'js_texts': js_texts}
 
 
 class MarkdownView(TextView):
@@ -325,7 +304,7 @@ class MarkdownView(TextView):
         c = b.get_variable_configuration(variable_definition)
         main_text = (
             f'<span id="{x.id}" '
-            f'class="_{self.mode_name} _{self.view_name} {self.variable_id}">'
+            f'class="_{x.mode_name} _{self.view_name} {self.variable_id}">'
             '</span>')
         if x.design_name not in ['none']:
             main_text = add_label_html(main_text, c, variable_id, element_id)
@@ -335,15 +314,10 @@ class MarkdownView(TextView):
             MARKDOWN_JS_OUTPUT.substitute({
                 'variable_id': variable_id,
                 'element_id': element_id,
-                'data_uri': data_uri,
-            }),
-        ]
+                'data_uri': data_uri})]
         return {
-            'css_uris': [],
-            'js_uris': self.js_uris,
-            'main_text': main_text,
-            'js_texts': js_texts,
-        }
+            'css_uris': [], 'js_uris': self.js_uris, 'main_text': main_text,
+            'js_texts': js_texts}
 
 
 class ImageView(VariableView):
@@ -358,9 +332,9 @@ class ImageView(VariableView):
         c = b.get_variable_configuration(variable_definition)
         main_text = (
             f'<img id="{x.id}" '
-            f'class="_{self.mode_name} _{self.view_name} {variable_id}" '
+            f'class="_{x.mode_name} _{self.view_name} {variable_id}" '
             f'src="{data_uri}" alt="">')
-        # TODO: Show spinner onerror
+        # TODO: Show spinner on error
         if x.design_name not in ['none']:
             main_text = add_label_html(main_text, c, variable_id, element_id)
         js_texts = [
@@ -368,18 +342,13 @@ class ImageView(VariableView):
             IMAGE_JS_OUTPUT.substitute({
                 'variable_id': variable_id,
                 'element_id': element_id,
-                'data_uri': data_uri,
-            }),
-        ]
+                'data_uri': data_uri})]
         return {
-            'css_uris': [],
-            'js_uris': [],
-            'main_text': main_text,
-            'js_texts': js_texts,
-        }
+            'css_uris': [], 'js_uris': [], 'main_text': main_text,
+            'js_texts': js_texts}
 
 
-class RadioView(StringView):
+class RadioView(VariableView):
 
     view_name = 'radio'
 
@@ -389,33 +358,65 @@ class RadioView(StringView):
         view_name = self.view_name
         element_id = x.id
         c = b.get_variable_configuration(variable_definition)
-        options = []
-        for d in c.get('options', []):
-            # TODO: Validate variable view configurations
-            option_value = d['value']
-            option_name = d.get('name', option_value)
-            option_id = format_slug(option_name)
-            options.append({
-                'id': option_id, 'name': option_name, 'value': option_value})
+        data = b.load_data_from(x.request_params, variable_definition)
+        value = data.get('value', '')
         main_text = RADIO_HTML_INPUT.render({
             'element_id': element_id,
-            'mode_name': self.mode_name,
+            'mode_name': x.mode_name,
             'view_name': view_name,
             'variable_id': variable_id,
-            'options': options,
-            'value': self.get_value(b),
-        })
+            'options': get_configuration_options(c, [value]),
+            'value': value})
         if x.design_name not in ['none']:
             main_text = add_label_html(main_text, c, variable_id, element_id)
         js_texts = [
-            RADIO_JS_INPUT.substitute({'view_name': view_name}),
-        ]
+            RADIO_JS_INPUT.substitute({'view_name': view_name})]
+        if variable_definition.step_name != 'input':
+            data_uri = b.get_data_uri(variable_definition, x)
+            js_texts.extend([
+                RADIO_JS_HEADER,
+                RADIO_JS_OUTPUT.substitute({
+                    'variable_id': variable_id, 'element_id': element_id,
+                    'data_uri': data_uri})])
         return {
-            'css_uris': [],
-            'js_uris': [],
-            'main_text': main_text,
-            'js_texts': js_texts,
-        }
+            'css_uris': [], 'js_uris': [], 'main_text': main_text,
+            'js_texts': js_texts}
+
+
+class CheckboxView(VariableView):
+
+    view_name = 'checkbox'
+
+    def render_input(self, b: Batch, x: Element):
+        element_id = x.id
+        view_name = self.view_name
+        variable_id = self.variable_id
+        variable_definition = self.variable_definition
+        c = b.get_variable_configuration(variable_definition)
+        data = b.load_data_from(x.request_params, variable_definition)
+        values = data.get('value', '').splitlines()
+        main_text = CHECKBOX_HTML_INPUT.render({
+            'element_id': element_id,
+            'mode_name': x.mode_name,
+            'view_name': view_name,
+            'variable_id': variable_id,
+            'options': get_configuration_options(c, values),
+            'values': values})
+        if x.design_name not in ['none']:
+            main_text = add_label_html(main_text, c, variable_id, element_id)
+        js_texts = [
+            CHECKBOX_JS_INPUT.substitute({'view_name': view_name})]
+        if variable_definition.step_name != 'input':
+            data_uri = b.get_data_uri(variable_definition, x)
+            js_texts.extend([
+                CHECKBOX_JS_HEADER,
+                CHECKBOX_JS_OUTPUT.substitute({
+                    'variable_id': variable_id,
+                    'element_id': element_id,
+                    'data_uri': data_uri})])
+        return {
+            'css_uris': [], 'js_uris': [], 'main_text': main_text,
+            'js_texts': js_texts}
 
 
 class TableView(VariableView):
@@ -430,7 +431,7 @@ class TableView(VariableView):
         c = b.get_variable_configuration(variable_definition)
         main_text = (
             f'<table id="{element_id}" '
-            f'class="_{self.mode_name} _{self.view_name} {variable_id}">'
+            f'class="_{x.mode_name} _{self.view_name} {variable_id}">'
             '<thead/><tbody/></table>')
         if x.design_name not in ['none']:
             main_text = add_label_html(main_text, c, variable_id, element_id)
@@ -439,15 +440,10 @@ class TableView(VariableView):
             TABLE_JS_OUTPUT.substitute({
                 'variable_id': variable_id,
                 'element_id': element_id,
-                'data_uri': data_uri,
-            }),
-        ]
+                'data_uri': data_uri})]
         return {
-            'css_uris': [],
-            'js_uris': [],
-            'main_text': main_text,
-            'js_texts': js_texts,
-        }
+            'css_uris': [], 'js_uris': [], 'main_text': main_text,
+            'js_texts': js_texts}
 
 
 class FrameView(VariableView):
@@ -460,14 +456,14 @@ class FrameView(VariableView):
         element_id = x.id
         data_uri = b.get_data_uri(variable_definition, x)
         c = b.get_variable_configuration(variable_definition)
-        data = b.get_data(variable_definition)
+        data = b.load_data(variable_definition)
         if 'value' in data:
             value = data['value']
         else:
             value = ''
         main_text = (
             f'<iframe id="{element_id}" '
-            f'class="_{self.mode_name} _{self.view_name} {variable_id}" '
+            f'class="_{x.mode_name} _{self.view_name} {variable_id}" '
             f'src="{escape_quotes_html(value)}" frameborder="0">'
             '</iframe>')
         if x.design_name not in ['none']:
@@ -478,15 +474,28 @@ class FrameView(VariableView):
             FRAME_JS_OUTPUT.substitute({
                 'variable_id': variable_id,
                 'element_id': element_id,
-                'data_uri': data_uri,
-            }),
-        ]
+                'data_uri': data_uri})]
         return {
-            'css_uris': [],
-            'js_uris': [],
-            'main_text': main_text,
-            'js_texts': js_texts,
-        }
+            'css_uris': [], 'js_uris': [], 'main_text': main_text,
+            'js_texts': js_texts}
+
+
+class JsonView(VariableView):
+
+    view_name = 'json'
+
+    def render_output(self, b: Batch, x: Element):
+        variable_definition = self.variable_definition
+        variable_id = self.variable_id
+        data_uri = b.get_data_uri(variable_definition, x)
+        js_texts = [
+            JSON_JS_HEADER,
+            JSON_JS_OUTPUT.substitute({
+                'variable_id': variable_id,
+                'data_uri': data_uri})]
+        return {
+            'css_uris': [], 'js_uris': [], 'main_text': '',
+            'js_texts': js_texts}
 
 
 def initialize_view_by_name():
@@ -570,6 +579,15 @@ def yield_data_by_id_from_txt(path, variable_definitions):
         raise CrossComputeConfigurationError(e)
 
 
+def get_data_from(request_params, variable_definition):
+    variable_id = variable_definition.id
+    if variable_id in request_params:
+        data = {'value': request_params[variable_id]}
+    else:
+        data = {}
+    return data
+
+
 def parse_data_by_id(data_by_id, variable_definitions):
     for variable_definition in variable_definitions:
         variable_id = variable_definition.id
@@ -589,28 +607,46 @@ def parse_data_by_id(data_by_id, variable_definitions):
     return data_by_id
 
 
-def update_variable_data(target_path, data_by_id):
-    target_path.parent.mkdir(parents=True, exist_ok=True)
+def update_variable_data(path, data_by_id):
+    path.parent.mkdir(parents=True, exist_ok=True)
     try:
-        if target_path.suffix == '.dictionary':
-            if target_path.exists():
-                with target_path.open('r+t') as f:
+        if path.suffix == '.dictionary':
+            if path.exists():
+                with path.open('r+t') as f:
                     d = json.load(f)
                     d.update(data_by_id)
                     f.seek(0)
-                    f.truncate()
                     json.dump(d, f)
+                    f.truncate()
             else:
-                with target_path.open('wt') as f:
+                with path.open('wt') as f:
                     d = data_by_id
                     json.dump(d, f)
         else:
-            with target_path.open('wt') as f:
+            with path.open('wt') as f:
                 f.write(data_by_id.values()[0])
     except (json.JSONDecodeError, OSError) as e:
         raise CrossComputeDataError(e)
 
 
+def remove_variable_data(path, variable_ids):
+    if not path.exists():
+        return
+    try:
+        if path.suffix == '.dictionary':
+            with path.open('r+t') as f:
+                d = json.load(f)
+                for variable_id in variable_ids:
+                    del d[variable_id]
+                f.seek(0)
+                json.dump(d, f)
+                f.truncate()
+        else:
+            path.unlink(missing_ok=True)
+    except (json.JSONDecodeError, OSError) as e:
+        raise CrossComputeDataError(e)
+
+
 def process_variable_data(path, variable_definition):
     variable_id = variable_definition.id
     variable_view = VariableView.get_from(variable_definition)
@@ -753,53 +789,72 @@ def get_label_text(variable_configuration, variable_id):
     return label_text.strip()
 
 
-def load_view_text(file_name):
-    return open(TEMPLATES_FOLDER / file_name).read().strip()
+def get_configuration_options(variable_configuration, variable_values):
+    options = []
+    for i, d in enumerate(variable_configuration.get('options', [{
+            'value': _} for _ in variable_values])):
+        option_value = d['value']
+        options.append({
+            'id': d.get('id', i),
+            'name': d.get('name', option_value),
+            'value': option_value})
+    return options
 
 
 L = getLogger(__name__)
 
 
-LINK_JS_HEADER = load_view_text('linkHeader.js')
-LINK_JS_OUTPUT = StringTemplate(load_view_text('linkOutput.js'))
+LINK_JS_HEADER = asset_storage.load_raw_text('link-header.js')
+LINK_JS_OUTPUT = asset_storage.load_string_text('link-output.js')
 
 
-STRING_HTML_INPUT = StringTemplate(load_view_text('stringInput.html'))
-STRING_JS_INPUT = StringTemplate(load_view_text('stringInput.js'))
-STRING_JS_HEADER = load_view_text('stringHeader.js')
-STRING_JS_OUTPUT = StringTemplate(load_view_text('stringOutput.js'))
+STRING_HTML_INPUT = asset_storage.load_jinja_text('string-input.html')
+STRING_JS_INPUT = asset_storage.load_string_text('string-input.js')
+STRING_JS_HEADER = asset_storage.load_raw_text('string-header.js')
+STRING_JS_OUTPUT = asset_storage.load_string_text('string-output.js')
 
 
-TEXT_HTML_INPUT = StringTemplate(load_view_text('textInput.html'))
-TEXT_JS_HEADER = load_view_text('textHeader.js')
-TEXT_JS_INPUT = StringTemplate(load_view_text('textInput.js'))
-TEXT_JS_OUTPUT = StringTemplate(load_view_text('textOutput.js'))
+TEXT_HTML_INPUT = asset_storage.load_string_text('text-input.html')
+TEXT_JS_HEADER = asset_storage.load_raw_text('text-header.js')
+TEXT_JS_INPUT = asset_storage.load_string_text('text-input.js')
+TEXT_JS_OUTPUT = asset_storage.load_string_text('text-output.js')
 
 
-MARKDOWN_JS_HEADER = load_view_text('markdownHeader.js')
-MARKDOWN_JS_OUTPUT = StringTemplate(load_view_text('markdownOutput.js'))
+MARKDOWN_JS_HEADER = asset_storage.load_raw_text('markdown-header.js')
+MARKDOWN_JS_OUTPUT = asset_storage.load_string_text('markdown-output.js')
 
 
-IMAGE_JS_HEADER = load_view_text('imageHeader.js')
-IMAGE_JS_OUTPUT = StringTemplate(load_view_text('imageOutput.js'))
+IMAGE_JS_HEADER = asset_storage.load_raw_text('image-header.js')
+IMAGE_JS_OUTPUT = asset_storage.load_string_text('image-output.js')
 
 
-RADIO_HTML_INPUT = JinjaTemplate(load_view_text('radioInput.html'))
-RADIO_JS_INPUT = StringTemplate(load_view_text('radioInput.js'))
+RADIO_HTML_INPUT = asset_storage.load_jinja_text('radio-input.html')
+RADIO_JS_HEADER = asset_storage.load_raw_text('radio-header.js')
+RADIO_JS_INPUT = asset_storage.load_string_text('radio-input.js')
+RADIO_JS_OUTPUT = asset_storage.load_string_text('radio-output.js')
 
 
-TABLE_JS_HEADER = load_view_text('tableHeader.js')
-TABLE_JS_OUTPUT = StringTemplate(load_view_text('tableOutput.js'))
+CHECKBOX_HTML_INPUT = asset_storage.load_jinja_text('checkbox-input.html')
+CHECKBOX_JS_HEADER = asset_storage.load_raw_text('checkbox-header.js')
+CHECKBOX_JS_INPUT = asset_storage.load_string_text('checkbox-input.js')
+CHECKBOX_JS_OUTPUT = asset_storage.load_string_text('checkbox-output.js')
 
 
-FRAME_JS_HEADER = load_view_text('frameHeader.js')
-FRAME_JS_OUTPUT = StringTemplate(load_view_text('frameOutput.js'))
+TABLE_JS_HEADER = asset_storage.load_raw_text('table-header.js')
+TABLE_JS_OUTPUT = asset_storage.load_string_text('table-output.js')
+
+
+FRAME_JS_HEADER = asset_storage.load_raw_text('frame-header.js')
+FRAME_JS_OUTPUT = asset_storage.load_string_text('frame-output.js')
+
+
+JSON_JS_HEADER = asset_storage.load_raw_text('json-header.js')
+JSON_JS_OUTPUT = asset_storage.load_string_text('json-output.js')
 
 
 YIELD_DATA_BY_ID_BY_EXTENSION = {
     '.csv': yield_data_by_id_from_csv,
-    '.txt': yield_data_by_id_from_txt,
-}
+    '.txt': yield_data_by_id_from_txt}
 
 
 FILE_DATA_CACHE = FileCache(
